unit MMultiViewer;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  fmxMRVMediaSource, fmxMRVCamera, FMX.Layouts, fmxMRVCamMultiView, fmxMRVType,
  FMX.Controls.Presentation, FMX.StdCtrls, fmxMRVCamView, fmxMRVCamRecorder,
  StrUtils, fmxMRVCamControl, FMX.ListBox, fmxMRVImg, fmxMRVBitmap,
  FMX.MultiView;

type
  TForm2 = class(TForm)
    btnConnect: TButton;
    btnDisconnect: TButton;
    WebCam: TCheckBox;
    Cam1: TRVCamera;
    Recorder1: TRVCamRecorder;
    ConnectVideo: TCheckBox;
    btnRecord: TButton;
    Viewer1: TRVCamView;
    Recorder2: TRVCamRecorder;
    RecordTimer: TTimer;
    btnSnapShot: TButton;
    SaveSnapShotDialog: TSaveDialog;
    LCounter: TLabel;
    procedure btnConnectClick(Sender: TObject);
    procedure Cam1SearchComplete(Sender: TObject; Status: Integer);
    procedure btnDisconnectClick(Sender: TObject);
    procedure btnRecordClick(Sender: TObject);
    procedure ConnectVideoChange(Sender: TObject);
    procedure RecordTimerTimer(Sender: TObject);
    procedure Cam1EndVideoFile(Sender: TObject; Status: Integer);
    procedure Cam1EndVideoStream(Sender: TObject; Status: Integer);
    procedure Cam1Downloaded(Sender: TObject; Status: Integer);
    procedure Cam1Download(Sender: TObject; Size, BytesRead: Integer);
    procedure Cam1StartVideoStream(Sender: TObject);
    procedure btnVertClick(Sender: TObject);
    procedure btnVertStopClick(Sender: TObject);
    procedure btnHorClick(Sender: TObject);
    procedure btnHorStopClick(Sender: TObject);
    procedure btnLEDOnClick(Sender: TObject);
    procedure btnLEDOffClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure cbFlipHorizontallyChange(Sender: TObject);
    procedure cbFlipVerticallyClick(Sender: TObject);
    procedure cmbResolutionChange(Sender: TObject);
    procedure cmbVideoModeChange(Sender: TObject);
    procedure tbBrightnessChange(Sender: TObject);
    procedure tbContrastChange(Sender: TObject);
    procedure btnSnapShotClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FVideoFormat: string;
    FActiveRecorder: TRVCamRecorder;
    FFileNameToDelete: string;

    FCounter: integer;
    FRecordingIntervals: integer;
    FRecorderInterlaceTime: integer;

    procedure RunCamera(RVCamera: TRVCamera);
    procedure SwitchRecorder(AActivateNextRecorder: Boolean);
    procedure StopCamera;
    procedure EnableDisableControls(ACam: TRVCamera);
    procedure LoadVideoResolutions(AComboBox: TComboBox);
    procedure LoadVideoFormats(ACam: TRVCamera; AComboBox: TComboBox);

    procedure FillCameraInfo(ACam: TRVCamera);
    procedure FillPTZInfo(ACam: TRVCamera);

    procedure SaveSnapShot(img: TRVMBitmap);

    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

implementation

{$R *.fmx}

procedure TForm2.FormCreate(Sender: TObject);
begin
  EnableDisableControls(Cam1);
  Panel1.Visible := False;
  MultiView1.Visible := False;
end;

procedure TForm2.btnConnectClick(Sender: TObject);

  procedure ConnectCamera(Cam: TRVCamera);
  begin
    Cam.CommandMode := rvsmNoWait;
    Cam.URL := '';
//    if WebCam.IsChecked then
//      Cam.CameraHost := 'integrated camera'
//    else
//    begin
      {Panasonic PTZ}
  //    Cam.CameraHost := 'www.nagatakensetsu.co.jp';
  //    Cam.CameraPort := 40080;
  //    Cam.UserName := '';
  //    Cam.UserPassword := '';

      {Panasonic China}
    Cam.CameraHost := 'cam6284208.miemasu.net';
    Cam.CameraPort := 0;
    Cam.UserName := '';
    Cam.UserPassword := '';

      {Integrated Camera}
  //  Cam.CameraHost := 'Integrated Camera';
  //  Cam.CameraPort := 0;
  //  Cam.UserName := '';
  //  Cam.UserPassword := '';
  //  WebCam.IsChecked := True;

    Cam.VideoImagesURLs.Clear;
//     end;
  //      s := C.VideoImagesURLs;
  //      Cam.CycleVideoImages := s <> '';
  //      while s <> '' do
  //      begin
  //        if Pos(';', s) > 0 then
  //          s0 := Copy(s, 1, Pos(';', s) - 1)
  //        else
  //          s0 := s;
  //        Delete(s, 1, length(s0) + 1);
  //        Cam.VideoImagesURLs.Add(s0);
  //      end;
  //  Cam.ToFile := True;
//    if WebCam.IsChecked then
//    begin
//      Cam.VideoDeviceIndex := 0;
//      Cam.DeviceType := rvdtWebCamera
//    end
//    else
    Cam.DeviceType := rvdtIPCamera;
    Cam.VideoFormat := rvvfMJPEG;
    Cam.FFMpegProperty.UseFFMpeg := False;
    Cam.GStreamerProperty.UseGStreamer := False;
  //        Cam.VideoMode := vm60HZ;            { vm60HZ, vm50HZ, vmOutdoor}

    {Configura Video Recorder}
  //  SwitchRecorder;

    { Execute Camera }
    if Cam.IsSearchCamera then
    begin
      Cam.Abort;
      Cam.WaitForSearch;
    end;

    if Cam.IsVideoPlaying then
    begin
      Cam.Abort;
      Cam.WaitForVideo;
      if Application.Terminated then
        Exit;
    end;

    if Cam.DeviceType = rvdtWebCamera then
      RunCamera(Cam)
    else
    if Cam.CameraHost <> '' then
    begin
      Cam.SearchCamera;
    end
    else
      RunCamera(Cam);
  end;

begin
  if Cam1.Aborting then
    Exit;

  FVideoFormat := 'avi';
  ConnectCamera(Cam1);
//  ConnectCamera(Cam2);


end;

procedure TForm2.Cam1Download(Sender: TObject; Size, BytesRead: Integer);
begin
  ShowMessage('Download');
end;

procedure TForm2.Cam1Downloaded(Sender: TObject; Status: Integer);
begin
  ShowMessage('Downloaded');
end;

procedure TForm2.Cam1EndVideoFile(Sender: TObject; Status: Integer);
begin
  ShowMessage('End File');
end;

procedure TForm2.Cam1StartVideoStream(Sender: TObject);
begin
  if FFileNameToDelete <> '' then
  begin
    DeleteFile(FFileNameToDelete);
    FFileNameToDelete := '';
  end;

  {Enable/Disable Controls and Fill Controls}
  EnableDisableControls(TRVCamera(Sender));
  LoadVideoResolutions(cmbResolution);
  LoadVideoFormats(TRVCamera(Sender), cmbVideoFormat);
  FillCameraInfo(TRVCamera(Sender));
end;

procedure TForm2.EnableDisableControls(ACam: TRVCamera);
var
  CamEnabled: Boolean;
  MoveEnabled, VideoEnabled: Boolean;
begin
  {Detect if Camera is Active}
  CamEnabled := assigned(ACam) and ACam.IsVideoPlaying;
  MoveEnabled := CamEnabled and (rvcm_Move in ACam.GetAccessibleCamMethods);
  VideoEnabled := CamEnabled and (rvcp_VideoResolution in ACam.GetAccessibleCamProperties);

  {General Controls}
  btnSnapShot.Enabled := CamEnabled;

  {Movement Controls}
  RVCamControl1.Enabled := MoveEnabled;
  btnStop.Enabled := MoveEnabled;
  btnVert.Enabled := MoveEnabled;
  btnVertStop.Enabled := MoveEnabled;
  btnHor.Enabled := MoveEnabled;
  btnHorStop.Enabled := MoveEnabled;
  btnLEDOn.Enabled := MoveEnabled;
  btnLEDOff.Enabled := MoveEnabled;
  cbFlipHorizontally.Enabled := MoveEnabled;
  cbFlipVertically.Enabled := MoveEnabled;

  {Video Controls}
  cmbVideoFormat.Enabled := VideoEnabled;
  cmbResolution.Enabled := VideoEnabled;
  cmbVideoMode.Enabled := VideoEnabled;
  tbBrightness.Enabled := VideoEnabled;
  tbContrast.Enabled := VideoEnabled;
end;

procedure TForm2.FillCameraInfo(ACam: TRVCamera);
begin
  // --- Control panel
  if ACam.VideoFormat = rvvfMJPEG then
    cmbVideoFormat.ItemIndex := 0
  else
    cmbVideoFormat.ItemIndex := 1;
  cmbResolution.ItemIndex := Ord(ACam.VideoResolution);
  cmbVideoMode.ItemIndex := Ord(ACam.VideoMode);
  cbFlipHorizontally.IsChecked := ACam.FlipHorizontally;
  cbFlipVertically.IsChecked := ACam.FlipVertically;
  tbBrightness.Value := ACam.Brightness;
  tbContrast.Value := ACam.Contrast;
//  // --- Page "Users"
//  FillUsers;
//  // --- Page "Parameters"
//  // Parameters | Information
//  with ACam.Parameters do
//  begin
//    lblDeviceID.Text := ID;
//    lblSys_Ver.Text := Version;
//    txtAlias.Text := Alias;
//    cbUPnPtoMapPort.IsChecked := UPnPtoMapPort;
//  end;
//  // Parameters | Network
//  with ACam.Parameters.Network do
//  begin
//    cbDynamicIP.IsChecked := DynamicIP;
//    txtIP.Text := IPAddr;
//    txtMask.Text := SubnetMask;
//    txtGateway.Text := Gateway;
//    txtDNS.Text := DNSServer;
//    txtHTTPPort.Text := HttpPort;
//  end;
//  // Parameters | Date&Time
//  with ACam.Parameters.DateTimeParameter do
//  begin
//    cbSyncWithPC.IsChecked := SyncWithPC;
//    txtNow.Text := DateTimeToStr(Now);
//    txtTimeZone.Text := IntToStr(TimeZone);
//    lblTimeZone.Text := GetTimeZoneStr;
//    cbNTPEnabled.IsChecked := NTPEnabled;
//    txtNTPServer.Text := NTPServer;
//  end;
//  // Parameters | WiFi
//  FillWIFIInfo;
//  // Parameters | ADSL
//  with ACam.Parameters.ADSL do
//  begin
//    cbUseADSL.IsChecked := IsUsed;
//    txtADSLUsername.Text := User;
//    txtADSLPassword.Text := Password;
//  end;
//  // Parameters | Alarm
//  FillAlarm;
//  // Parameters | Mail
//  FillMail;
//  // Parameters | FTP
//  FillFtp;
//  // Parameters | Video decoder
//  txtBitrate.Text := IntToStr(ACam.Parameters.Decoder.Bitrate);
//  txtFramePerSec.Text := IntToStr(ACam.FramePerSecInt);

  // Parameters | Pan-tilt-zoom
  FillPTZInfo(ACam);
end;

procedure TForm2.FillPTZInfo(ACam: TRVCamera);
begin
//  with ACam.Parameters.PTZSettings do
//  begin
//    cmbLedMode.ItemIndex := Ord(LedMode);
//    cbCenterOnStart.IsChecked := CenterOnStart;
//    txtAutoPatrolInterval.Text := IntToStr(AutoPatrolInterval);
//    cmbAutoPatrolType.ItemIndex := Ord(AutoPatrolType);
//    txtPatrolHoriz.Text := IntToStr(PatrolHoriz);
//    txtPatrolVert.Text := IntToStr(PatrolVert);
//    sbPatrolRate.Value := PatrolRate;
//    sbPatrolUpRate.Value := PatrolUpRate;
//    sbPatrolDownRate.Value := PatrolDownRate;
//    sbPatrolLeftRate.Value := PatrolLeftRate;
//    sbPatrolRightRate.Value := PatrolRightRate;
//  end;
end;

procedure TForm2.Cam1EndVideoStream(Sender: TObject; Status: Integer);
var
  f: file of Byte;
  size: Longint;
  AFileName: string;
begin
  if assigned(FActiveRecorder) then
  begin
    AFileName := FActiveRecorder.OutputFileName;
    AssignFile(f, AFileName);
    Reset(f);
    try
      size := FileSize(f);
      if size < 10240 then {If size less than 10MB remove file }
        FFileNameToDelete := AFileName;

    finally
      CloseFile(f);
    end;
  end;
end;

procedure TForm2.Cam1SearchComplete(Sender: TObject; Status: Integer);
begin
  if Status = 0 then
  begin
    RunCamera(TRVCamera(Sender));
  end;
end;

procedure TForm2.ConnectVideoChange(Sender: TObject);
begin
  if ConnectVideo.IsChecked then
  begin
    Viewer1.VideoSource := Cam1;
  end
  else
  begin
    Viewer1.VideoSource := nil;
  end;
end;

procedure TForm2.RecordTimerTimer(Sender: TObject);
begin
  inc(FCounter);
  LCounter.Text := 'Counter: '+FCounter.ToString;

  if FCounter = FRecordingIntervals then    {Inactivates Active Recorder}
  begin
    FCounter := 0;
    SwitchRecorder(False);
  end
  else                                      {Activates Next Recorder}
  if ((FCounter + FRecorderInterlaceTime) = FRecordingIntervals) then
  begin
    SwitchRecorder(True);
  end
  else
  begin
    if not Cam1.IsVideoPlaying then
      Cam1.PlayVideoStream;
//    if not Cam2.IsVideoPlaying then
//      Cam2.PlayVideoStream;
  end;
end;

procedure TForm2.RunCamera(RVCamera: TRVCamera);
begin
  if RVCamera.IsVideoPlaying then
  begin
    RVCamera.Abort;
    RVCamera.WaitForVideo;
    if Application.Terminated then
      Exit;
  end;
  RVCamera.PlayVideoStream;
end;

procedure TForm2.btnRecordClick(Sender: TObject);
begin
  {Clocks ticks each 1/2 second        }
  {so needs to multiplay intervals by 2}
  FRecordingIntervals := 60; {60 Seconds}
  FRecordingIntervals := (FRecordingIntervals * 2);

  FRecorderInterlaceTime := 1; {1 Second}
  FRecorderInterlaceTime := (FRecorderInterlaceTime * 2);

  FCounter := 0;

  if not RecordTimer.Enabled then
    SwitchRecorder(True);
  RecordTimer.Enabled := True;
end;

procedure TForm2.btnSnapShotClick(Sender: TObject);
var
  img: TRVImageWrapper;
begin
  if not Cam1.IsVideoPlaying then
    Exit;

  img := Cam1.GetSnapShot;
  try
    SaveSnapShot(img.Bitmap);
  finally
    img.Free;
  end;
end;

procedure TForm2.SaveSnapShot(img: TRVMBitmap);
var
  jpg: TBitmap;
begin
  if (img <> nil) and SaveSnapShotDialog.Execute then
  begin
    jpg := TBitmap.Create;
    try
      jpg.Assign(img.GetBitmap);
      jpg.SaveToFile(SaveSnapShotDialog.FileName);
    finally
      jpg.Free;
    end;
  end;
end;

procedure TForm2.btnStopClick(Sender: TObject);
begin
  Cam1.MoveStop;
end;

procedure TForm2.btnVertClick(Sender: TObject);
begin
  Cam1.MoveVPatrol;
end;

procedure TForm2.btnVertStopClick(Sender: TObject);
begin
  Cam1.MoveVPatrolStop;
end;

procedure TForm2.btnHorClick(Sender: TObject);
begin
  Cam1.MoveHPatrol;
end;


procedure TForm2.btnHorStopClick(Sender: TObject);
begin
  Cam1.MoveHPatrolStop;
end;

procedure TForm2.btnLEDOffClick(Sender: TObject);
begin
  Cam1.SwitchLEDOff;
end;

procedure TForm2.btnLEDOnClick(Sender: TObject);
begin
  Cam1.SwitchLEDOn;
end;

procedure TForm2.cbFlipHorizontallyChange(Sender: TObject);
begin
  Cam1.FlipHorizontally := cbFlipHorizontally.IsChecked;
end;

procedure TForm2.cbFlipVerticallyClick(Sender: TObject);
begin
  Cam1.FlipVertically := cbFlipVertically.IsChecked;
end;

procedure TForm2.cmbResolutionChange(Sender: TObject);
begin
  if cmbResolution.ItemIndex < 0 then
    Exit;

  Cam1.VideoResolution := TRVVideoResolution(cmbResolution.ItemIndex);
end;

procedure TForm2.cmbVideoModeChange(Sender: TObject);
begin
  case cmbVideoMode.ItemIndex of
    0: Cam1.VideoMode := vm50HZ;
    1: Cam1.VideoMode := vm60HZ;
    2: Cam1.VideoMode := vmOutdoor;
  end;
end;

procedure TForm2.tbBrightnessChange(Sender: TObject);
begin
  Cam1.Brightness := Round(tbBrightness.Value);
end;

procedure TForm2.tbContrastChange(Sender: TObject);
begin
  Cam1.Contrast := Round(tbContrast.Value);
end;

procedure TForm2.btnDisconnectClick(Sender: TObject);
begin
  if Cam1.Aborting then
    Exit;

  {Stop Timer}
  RecordTimer.Enabled := False;

  StopCamera;
end;

procedure TForm2.StopCamera;

  procedure StoppingCamera(Cam: TRVCamera);
  begin
    if Cam.IsSearchCamera then
    begin
      Cam.Abort;
      Cam.WaitForSearch;
    end;

    if Cam.IsVideoPlaying then
    begin
      Cam.Abort;
      Cam.WaitForVideo;
      if Application.Terminated then
        Exit;
    end;
  end;

begin
  {Stop Recording}
  if assigned(FActiveRecorder) then
  begin
    FActiveRecorder.Active := False;
//    FActiveRecorder.VideoSource := nil;
//    FActiveRecorder.AudioSource := nil;
  end;

  { Stop Camera }
  StoppingCamera(Cam1);
//  StoppingCamera(Cam2);
end;

procedure TForm2.SwitchRecorder(AActivateNextRecorder: Boolean);

  function CreateFileName: string;
  var
    wDateTime: string;
  begin
    wDateTime := formatdatetime('yyyy/mm/dd hh:nn:ss', Now);
    wDateTime := AnsiReplaceStr(wDateTime, ' ', '_');
    wDateTime := AnsiReplaceStr(wDateTime, '.', '');
    wDateTime := AnsiReplaceStr(wDateTime, ':', '');
    wDateTime := AnsiReplaceStr(wDateTime, '/', '');

    Result := 'video'+wDateTime+'.'+FVideoFormat;
  end;

  procedure ActivateRecorder(ARecorder: TRVCamRecorder);
  var
    AFileName: string;
  begin
    AFileName := CreateFileName;

    ARecorder.OutputFileName := AFileName;
    ARecorder.UseAudio := False;
    ARecorder.UseVideo := True;
    ARecorder.VideoCodec := rvvcDefault;
    ARecorder.VideoFramePerSec := 25;
    ARecorder.VideoBitrate := 800000;
    ARecorder.Active := True;
    if not ARecorder.Active then
    begin
      ShowMessage('Failed to initialize recording: '+AFileName);
    end;
  end;

  procedure InactivateRecorder(ARecorder: TRVCamRecorder);
  begin
    if assigned(ARecorder)
    and ARecorder.Active then
    begin
      ARecorder.Active := False;
    end;

  end;

begin
  if AActivateNextRecorder then
  begin
    if not assigned(FActiveRecorder)
    or (FActiveRecorder = Recorder2) then
    begin
      ActivateRecorder(Recorder1);
      FActiveRecorder := Recorder1;
    end
    else
    if FActiveRecorder = Recorder1 then
    begin
      ActivateRecorder(Recorder2);
      FActiveRecorder := Recorder2;
    end
  end
  else
  begin
    if (FActiveRecorder = Recorder2) then
      InactivateRecorder(Recorder1)
    else
    if FActiveRecorder = Recorder1 then
      InactivateRecorder(Recorder2);
  end;
  if not Cam1.IsVideoPlaying then
    Cam1.PlayVideoStream;
//  if not Cam2.IsVideoPlaying then
//    Cam2.PlayVideoStream;
end;

procedure TForm2.LoadVideoResolutions(AComboBox: TComboBox);
begin
  AComboBox.Items.Clear;
  AComboBox.Items.Add('Default');
  AComboBox.Items.Add('160 x 120');
  AComboBox.Items.Add('320 x 240');
  AComboBox.Items.Add('640 x 480');
  AComboBox.Items.Add('1024 x 768');
  AComboBox.Items.Add('1280 x 720');
  AComboBox.Items.Add('1900 x 1280');
end;

procedure TForm2.LoadVideoFormats(ACam: TRVCamera; AComboBox: TComboBox);
begin
  AComboBox.Items.Clear;
  AComboBox.Items.Add('MJPEG');
  if Cam1.IsSupportedFFMPEG then
    cmbVideoFormat.Items.Add('H264');
end;

end.
